/* 
 * File:   main.c
 * Author: Zdeněk Žamberský
 *
 * Created on July 11, 2016, 5:09 PM
 */

#include <stdio.h>

#include <glib.h>
#include <libsecret/secret.h>
#include <stdlib.h>
#include <string.h>

/*
 * 
 */
 

SecretSchema reproducer_schema = {
    "reproducer.schema", SECRET_SCHEMA_NONE,
    {
        { "username", SECRET_SCHEMA_ATTRIBUTE_STRING},
        { "url", SECRET_SCHEMA_ATTRIBUTE_STRING},
        { "NULL", 0}
    }
};

void checkForError(char* name, gboolean success, GError *error);
void setPassword(char* url, char* username, gchar *gpassword, char* descrition);
gchar * getPassword(char* url, char* username);
void clearPassword(char* url, char* username);

int wasError;

void checkForError(char* name, gboolean success, GError *error) {
    if (!success || error != NULL) {
        gchar* errorMessage = ( error == NULL ) ? "NULL" :  error->message;
        fprintf(stderr, "\nREPRODUCER: error in %s:\n  success (by returned value): %d\n  GError object: %s\n\n", 
                name, success, errorMessage);
        wasError=1;
    }
}

void setPassword(char *url, char *username, gchar *gpassword, char *description) {
    GError *error = NULL;
    gboolean returned;
    returned = secret_password_store_sync(&reproducer_schema, 
            SECRET_COLLECTION_DEFAULT, description, gpassword, NULL, &error, 
            "username", username, "url", url, NULL);
    checkForError("secret_password_store_sync", returned, error);
}

gchar * getPassword(char *url, char *username) {
    GError *error = NULL;
    gchar *password = NULL;
    password = secret_password_lookup_sync(&reproducer_schema, NULL, &error,
            "username", username, "url", url, NULL);
    checkForError("secret_password_lookup_sync", password != NULL, error);
    return password;
}

void clearPassword(char* url, char* username) {
    GError *error = NULL;
    gboolean returned;
    returned = secret_password_clear_sync(&reproducer_schema, NULL, &error,
            "username", username, "url", url, NULL);
    checkForError("secret_password_clear_sync", returned, error);
}

int main(int argc, char **argv) {
    char *url = "http://reproducer";
    char *username = "username";
    gchar *password = "password";
    char *description = "This is reproducer";

    gchar *readPassword=NULL;
    
    wasError=0;

    setPassword(url, username, password, description);
    readPassword = getPassword(url, username);
    if(readPassword == NULL) {
        fprintf(stderr, "\nREPRODUCER: returned password is null\n", readPassword, password );
    	wasError=1;
    } else if(strcmp(password,readPassword) !=0 ) {
    	fprintf(stderr, "\nREPRODUCER: passwords are not equal ( %s != %s )\n", readPassword, password );
    	wasError=1;
    }
    secret_password_free(readPassword);
    clearPassword(url, username);

    if (wasError != 0)
    	return 1;
    return (EXIT_SUCCESS);
}

